<?php
/**
 * @file
 * jQuad's Core Functions
 */

function jquad_allowed_forms($access = TRUE) {
  global $user;
  $forms = array();
  // form_id => menu_path
  if ($access) {
    if (user_is_anonymous() && (bool) variable_get('jquad_login', TRUE)) {
      $forms['user_login'] = 'user';
    }
    if (user_register_access() && (bool) variable_get('jquad_register', TRUE)) {
      $forms['user_register'] = 'user/register';
    }
    if (user_is_anonymous() && (bool) variable_get('jquad_password', TRUE)) {
      $forms['user_pass'] = 'user/password';
    }
    if (user_is_logged_in() && (bool) variable_get('jquad_logout', TRUE)) {
      $forms['logout'] = 'logout';
    }
  }
  else {
    $forms = array(
      'user_login' => 'user/login',
      'user_register' => 'user/register',
      'user_pass' => 'user/password',
      'logout' => 'logout',
    );
  }
  // @TODO: Process multi-step forms for user account edits
  // if ($user->uid) {
  //   $forms['user_edit'] = 'user/' . $user->uid . '/edit';
  // }
  return $forms;
}

function jquad_generate_paths($access = TRUE) {
  $support = jquad_support();
  // Generate internal, relative, absolute and, if needed, secure paths.
  $paths = array(
    'current' => $_GET['q'],
    'jquad' => 'ajax/jquad',
    'throbber' => theme('jquad_throbber'),
  );
  $paths = array_merge($paths, jquad_allowed_forms($access));
  foreach ($paths as $key => $path) {
    // Relative paths
    $paths[$key] = array(
      'path' => $path,
      'relative' => url($path),
    );
    // Paths could be secure, check for SSL
    if ($support['securepages']) {
      $paths[$key]['secure'] = (bool) securepages_match($path);
      $paths[$key]['absolute'] = securepages_url($path, array('absolute' => TRUE, 'secure' => $paths[$key]['secure']));
    }
    // Paths do not use SSL
    else {
      $paths[$key]['secure'] = FALSE;
      $paths[$key]['absolute'] = url($path, array('absolute' => TRUE));
    }
  }
  return $paths;
}

/**
 * Get the supported data of a specific module or all supported modules.
 * The returned data will include any modifications made by any module that implements hook_schema_alter().
 *
 * @param $module
 *   - The name of the module to return support data. If not given, all supported module data is returned.
 *
 * @param $rebuild
 *   - If true, the data will be rebuilt instead of retrieved from the cache.
 *
 * @return
 *   An array of supported module data, empty if no modules are supported.
 *   FALSE if $module is provided and is not supported.
 */
function jquad_support($module = NULL, $rebuild = FALSE) {
  static $modules = array();
  if (empty($modules) || $rebuild) {
    // Attempt to load cached data if not rebuilding.
    if (!$rebuild && $cached = cache_get('jquad_support')) {
      $modules = $cached->data;
    }
    // Otherwise, rebuild data.
    else {
      $modules = array();
      // Invoke hook_jquad_support for all modules.
      foreach (module_implements('jquad_support') as $module) {
        $invoke = (array) module_invoke($module, 'jquad_support');
        $modules = array_merge($modules, $invoke);
      }
      // Allow all modules to alter the data
      drupal_alter('jquad_support', $modules);
      // Cache the data
      cache_set('jquad_support', $modules);
    }
  }
  if (!isset($module)) {
    return $modules;
  }
  elseif (isset($modules[$module])) {
    return $modules[$module];
  }
  else {
    return FALSE;
  }
}

/**
 * Implementation of hook_jquad_support().
 */
function jquad_jquad_support() {
  $module_path = drupal_get_path('module', 'jquad');
  $modules = array(
    'user' => array(
      'include' => './' . $module_path . '/includes/user.inc',
      'validate' => 'user_login_authenticate_validate',
    ),
  );
  if (module_exists('openid')) {
    $modules['openid'] = array(
      'validate' => 'openid_login_validate',
      'include' => './' . $module_path . '/includes/openid.inc',
      'js' => $module_path . '/js/openid.js',
      'css' => array(
        drupal_get_path('module', 'openid') . '/openid.css',
        $module_path . '/css/openid.css',
      ),
    );
  }
  if (module_exists('securepages')) {
    $modules['securepages'] = array(
      // Only load the jQuery AJAX XSS (http -> https) fix if securepages is used.
      // http://friedcellcollective.net/outbreak/jsjquerywindownameplugin/
      'js' => $module_path . '/js/jquery.windowname.js',
    );
  }
  return $modules;
}

function jquad_md5_conflicts() {
  static $conflicting_module = array();
  $modules = array(
    'ldapauth', // LDAP Integration module needs clear-text passwords to be sent to authenticate with servers
  );
  foreach ($modules as $module) {
    if (!isset($conflicting_module[$module])) {
      if (module_exists($module)) {
        $info = drupal_parse_info_file(drupal_get_path('module', $module) . '/' . $module . '.info');
        $conflicting_module[$module] = '<strong>' . t($info['name']) . '</strong> (' . $module . ')';
        break;
      }
    }
  }
  return $conflicting_module;
}

function jquad_to_js($var) {
  switch (gettype($var)) {
    case 'boolean':
      return $var ? 'true' : 'false'; // Lowercase necessary!
    case 'integer':
    case 'double':
      return $var;
    case 'resource':
    case 'string':
      return '"'. str_replace(array("\n", "\r", "<",      ">",      "&",      "'",      '"'),
                              array('\\\n', '\\\r', '\\\u003c', '\\\u003e', '\\\u0026', '\\\u0027', '\\\u0022'),
                              $var) .'"';
    case 'array':
      // Arrays in JSON can't be associative. If the array is empty or if it
      // has sequential whole number keys starting with 0, it's not associative
      // so we can go ahead and convert it as an array.
      if (empty ($var) || array_keys($var) === range(0, sizeof($var) - 1)) {
        $output = array();
        foreach ($var as $v) {
          $output[] = jquad_to_js($v);
        }
        return '[ '. implode(', ', $output) .' ]';
      }
      // Otherwise, fall through to convert the array as an object.
    case 'object':
      $output = array();
      foreach ($var as $k => $v) {
        $output[] = jquad_to_js(strval($k)) .': '. jquad_to_js($v);
      }
      return '{ '. implode(', ', $output) .' }';
    default:
      return 'null';
  }
}
